@@ -77,12 +77,12 @@ get_next_offset (uint64_t *offset, uint64_t *count)
   return r;
 }
 
-static void *worker_thread (void *ip);
+static void *worker_thread (void *wp);
 
 void
 multi_thread_copying (void)
 {
-  pthread_t *workers;
+  struct worker *workers;
   size_t i;
   int err;
 
@@ -99,16 +99,17 @@ multi_thread_copying (void)
 */
   assert (src->size != -1);
 
-  workers = malloc (sizeof (pthread_t) * threads);
+  workers = calloc (threads, sizeof *workers);
   if (workers == NULL) {
-    perror ("malloc");
+    perror ("calloc");
     exit (EXIT_FAILURE);
   }
 
   /* Start the worker threads. */
   for (i = 0; i < threads; ++i) {
-    err = pthread_create (&workers[i], NULL, worker_thread,
-                          (void *)(uintptr_t)i);
+    workers[i].index = i;
+    err = pthread_create (&workers[i].thread, NULL, worker_thread,
+                          &workers[i]);
     if (err != 0) {
       errno = err;
       perror ("pthread_create");
@@ -118,7 +119,7 @@ multi_thread_copying (void)
 
   /* Wait until all worker threads exit. */
   for (i = 0; i < threads; ++i) {
-    err = pthread_join (workers[i], NULL);
+    err = pthread_join (workers[i].thread, NULL);
     if (err != 0) {
       errno = err;
       perror ("pthread_join");
@@ -129,116 +130,191 @@ multi_thread_copying (void)
   free (workers);
 }
 
-static void wait_for_request_slots (uintptr_t index);
-static unsigned in_flight (uintptr_t index);
-static void poll_both_ends (uintptr_t index);
+static void wait_for_request_slots (struct worker *worker);
+static unsigned in_flight (size_t index);
+static void poll_both_ends (size_t index);
 static int finished_read (void *vp, int *error);
-static int free_command (void *vp, int *error);
+static int finished_command (void *vp, int *error);
+static void free_command (struct command *command);
 static void fill_dst_range_with_zeroes (struct command *command);
+static struct command *create_command (uint64_t offset, size_t len, bool zero,
+                                       struct worker *worker);
+
+/* Tracking worker queue size.
+ *
+ * The queue size is increased when starting a read command.
+ *
+ * The queue size is decreased when a read command is converted to zero
+ * subcommand in finished_read(), or when a write command completes in
+ * finished_command().
+ *
+ * Zero commands are not considered in the queue size since they have no
+ * payload.
+ */
+
+static inline void
+increase_queue_size (struct worker *worker, size_t len)
+{
+  assert (worker->queue_size < queue_size);
+  worker->queue_size += len;
+}
+
+static inline void
+decrease_queue_size (struct worker *worker, size_t len)
+{
+  assert (worker->queue_size >= len);
+  worker->queue_size -= len;
+}
+
+/* Using the extents map 'exts', check if the region
+ * [offset..offset+len-1] intersects only with zero extents.
+ *
+ * The invariant for '*i' is always an extent which starts before or
+ * equal to the current offset.
+ */
+static bool
+only_zeroes (const extent_list exts, size_t *i,
+             uint64_t offset, unsigned len)
+{
+  size_t j;
+
+  /* Invariant. */
+  assert (*i < exts.len);
+  assert (exts.ptr[*i].offset <= offset);
+
+  /* Update the invariant.  Search for the last possible extent in the
+   * list which is <= offset.
+   */
+  for (j = *i + 1; j < exts.len; ++j) {
+    if (exts.ptr[j].offset <= offset)
+      *i = j;
+    else
+      break;
+  }
+
+  /* Check invariant again. */
+  assert (*i < exts.len);
+  assert (exts.ptr[*i].offset <= offset);
+
+  /* If *i is not the last extent, then the next extent starts
+   * strictly beyond our current offset.
+   */
+  assert (*i == exts.len - 1 || exts.ptr[*i + 1].offset > offset);
+
+  /* Search forward, look for any non-zero extents overlapping the region. */
+  for (j = *i; j < exts.len; ++j) {
+    uint64_t start, end;
+
+    /* [start..end-1] is the current extent. */
+    start = exts.ptr[j].offset;
+    end = exts.ptr[j].offset + exts.ptr[j].length;
+
+    assert (end > offset);
+
+    if (start >= offset + len)
+      break;
+
+    /* Non-zero extent covering this region => test failed. */
+    if (!exts.ptr[j].zero)
+      return false;
+  }
+
+  return true;
+}
 
 /* There are 'threads' worker threads, each copying work ranges from
  * src to dst until there are no more work ranges.
  */
 static void *
-worker_thread (void *indexp)
+worker_thread (void *wp)
 {
-  uintptr_t index = (uintptr_t) indexp;
+  struct worker *w = wp;
   uint64_t offset, count;
   extent_list exts = empty_vector;
 
   while (get_next_offset (&offset, &count)) {
-    size_t i;
+    struct command *command;
+    size_t extent_index;
+    bool is_zeroing = false;
+    uint64_t zeroing_start = 0; /* initialized to avoid bogus GCC warning */
 
     assert (0 < count && count <= THREAD_WORK_SIZE);
     if (extents)
-      src->ops->get_extents (src, index, offset, count, &exts);
+      src->ops->get_extents (src, w->index, offset, count, &exts);
     else
-      default_get_extents (src, index, offset, count, &exts);
+      default_get_extents (src, w->index, offset, count, &exts);
 
-    for (i = 0; i < exts.size; ++i) {
-      struct command *command;
-      struct buffer *buffer;
-      char *data;
-      size_t len;
+    extent_index = 0; // index into extents array used to optimize only_zeroes
+    while (count) {
+      const size_t len = MIN (count, request_size);
 
-      if (exts.ptr[i].zero) {
+      if (only_zeroes (exts, &extent_index, offset, len)) {
         /* The source is zero so we can proceed directly to skipping,
-         * fast zeroing, or writing zeroes at the destination.
+         * fast zeroing, or writing zeroes at the destination.  Defer
+         * zeroing so we can send it as a single large command.
          */
-        command = calloc (1, sizeof *command);
-        if (command == NULL) {
-          perror ("malloc");
-          exit (EXIT_FAILURE);
+        if (!is_zeroing) {
+          is_zeroing = true;
+          zeroing_start = offset;
         }
-        command->offset = exts.ptr[i].offset;
-        command->slice.len = exts.ptr[i].length;
-        command->slice.base = 0;
-        command->index = index;
-        fill_dst_range_with_zeroes (command);
       }
-
       else /* data */ {
-        /* As the extent might be larger than permitted for a single
-         * command, we may have to split this into multiple read
-         * requests.
-         */
-        while (exts.ptr[i].length > 0) {
-          len = exts.ptr[i].length;
-          if (len > request_size)
-            len = request_size;
-          data = malloc (len);
-          if (data == NULL) {
-            perror ("malloc");
-            exit (EXIT_FAILURE);
-          }
-          buffer = calloc (1, sizeof *buffer);
-          if (buffer == NULL) {
-            perror ("malloc");
-            exit (EXIT_FAILURE);
-          }
-          buffer->data = data;
-          buffer->refs = 1;
-          command = calloc (1, sizeof *command);
-          if (command == NULL) {
-            perror ("malloc");
-            exit (EXIT_FAILURE);
-          }
-          command->offset = exts.ptr[i].offset;
-          command->slice.len = len;
-          command->slice.base = 0;
-          command->slice.buffer = buffer;
-          command->index = index;
-
-          wait_for_request_slots (index);
-
-          /* Begin the asynch read operation. */
-          src->ops->asynch_read (src, command,
-                                 (nbd_completion_callback) {
-                                   .callback = finished_read,
-                                   .user_data = command,
-                                 });
-
-          exts.ptr[i].offset += len;
-          exts.ptr[i].length -= len;
+        /* If we were in the middle of deferred zeroing, do it now. */
+        if (is_zeroing) {
+          /* Note that offset-zeroing_start can never exceed
+           * THREAD_WORK_SIZE, so there is no danger of overflowing
+           * size_t.
+           */
+          command = create_command (zeroing_start, offset-zeroing_start,
+                                    true, w);
+          fill_dst_range_with_zeroes (command);
+          is_zeroing = false;
         }
+
+        /* Issue the asynchronous read command. */
+        command = create_command (offset, len, false, w);
+
+        wait_for_request_slots (w);
+
+        /* NOTE: Must increase the queue size after waiting. */
+        increase_queue_size (w, len);
+
+        /* Begin the asynch read operation. */
+        src->ops->asynch_read (src, command,
+                               (nbd_completion_callback) {
+                                 .callback = finished_read,
+                                 .user_data = command,
+                               });
       }
 
-      offset += count;
-      count = 0;
-    } /* for extents */
+      offset += len;
+      count -= len;
+    } /* while (count) */
+
+    /* If we were in the middle of deferred zeroing, do it now. */
+    if (is_zeroing) {
+      /* Note that offset-zeroing_start can never exceed
+       * THREAD_WORK_SIZE, so there is no danger of overflowing
+       * size_t.
+       */
+      command = create_command (zeroing_start, offset - zeroing_start,
+                                true, w);
+      fill_dst_range_with_zeroes (command);
+      //is_zeroing = false;
+    }
   }
 
   /* Wait for in flight NBD requests to finish. */
-  while (in_flight (index) > 0)
-    poll_both_ends (index);
+  while (in_flight (w->index) > 0)
+    poll_both_ends (w->index);
 
   free (exts.ptr);
   return NULL;
 }
 
-/* If the number of requests in flight exceeds the limit, poll
- * waiting for at least one request to finish.  This enforces
- * the user --requests option.
+/* If the number of requests or queued bytes in flight exceed limits,
+ * then poll until enough requests finish.  This enforces the user
+ * --requests and --queue-size options.
  *
  * NB: Unfortunately it's not possible to call this from a callback,
  * since it will deadlock trying to grab the libnbd handle lock.  This
@@ -248,15 +324,16 @@ worker_thread (void *indexp)
  * limit. XXX
  */
 static void
-wait_for_request_slots (uintptr_t index)
+wait_for_request_slots (struct worker *worker)
 {
-  while (in_flight (index) >= max_requests)
-    poll_both_ends (index);
+  while (in_flight (worker->index) >= max_requests ||
+         worker->queue_size >= queue_size)
+    poll_both_ends (worker->index);
 }
 
 /* Count the number of asynchronous commands in flight. */
 static unsigned
-in_flight (uintptr_t index)
+in_flight (size_t index)
 {
   return src->ops->in_flight (src, index) + dst->ops->in_flight (dst, index);
 }
@@ -265,7 +342,7 @@ in_flight (uintptr_t index)
  * along.  This is a lightly modified nbd_poll.
  */
 static void
-poll_both_ends (uintptr_t index)
+poll_both_ends (size_t index)
 {
   struct pollfd fds[2];
   int r, direction;
@@ -338,13 +415,58 @@ poll_both_ends (uintptr_t index)
   }
 }
 
+/* Create a new buffer. */
+static struct buffer*
+create_buffer (size_t len)
+{
+  struct buffer *buffer;
+
+  buffer = calloc (1, sizeof *buffer);
+  if (buffer == NULL) {
+    perror ("calloc");
+    exit (EXIT_FAILURE);
+  }
+
+  buffer->data = malloc (len);
+  if (buffer->data == NULL) {
+    perror ("malloc");
+    exit (EXIT_FAILURE);
+  }
+
+  buffer->refs = 1;
+
+  return buffer;
+}
+
+/* Create a new command for read or zero. */
+static struct command *
+create_command (uint64_t offset, size_t len, bool zero, struct worker *worker)
+{
+  struct command *command;
+
+  command = calloc (1, sizeof *command);
+  if (command == NULL) {
+    perror ("calloc");
+    exit (EXIT_FAILURE);
+  }
+
+  command->offset = offset;
+  command->slice.len = len;
+
+  if (!zero)
+    command->slice.buffer = create_buffer (len);
+
+  command->worker = worker;
+
+  return command;
+}
+
 /* Create a sub-command of an existing command.  This creates a slice
- * referencing the buffer of the existing command in order to avoid
- * copying.
+ * referencing the buffer of the existing command without copying.
  */
 static struct command *
-copy_subcommand (struct command *command, uint64_t offset, size_t len,
-                 bool hole)
+create_subcommand (struct command *command, uint64_t offset, size_t len,
+                   bool zero)
 {
   const uint64_t end = command->offset + command->slice.len;
   struct command *newcommand;
@@ -359,12 +481,12 @@ copy_subcommand (struct command *command, uint64_t offset, size_t len,
   }
   newcommand->offset = offset;
   newcommand->slice.len = len;
-  if (!hole) {
+  if (!zero) {
     newcommand->slice.buffer = command->slice.buffer;
     newcommand->slice.buffer->refs++;
     newcommand->slice.base = offset - command->offset;
   }
-  newcommand->index = command->index;
+  newcommand->worker = command->worker;
 
   return newcommand;
 }
@@ -378,8 +500,8 @@ finished_read (void *vp, int *error)
   struct command *command = vp;
 
   if (*error) {
-    fprintf (stderr, "read at offset %" PRId64 " failed: %s\n",
-             command->offset, strerror (*error));
+    fprintf (stderr, "%s: read at offset %" PRId64 " failed: %s\n",
+             prog, command->offset, strerror (*error));
     exit (EXIT_FAILURE);
   }
 
@@ -389,7 +511,7 @@ finished_read (void *vp, int *error)
      */
     dst->ops->asynch_write (dst, command,
                             (nbd_completion_callback) {
-                              .callback = free_command,
+                              .callback = finished_command,
                               .user_data = command,
                             });
   }
@@ -397,10 +519,9 @@ finished_read (void *vp, int *error)
     const uint64_t start = command->offset;
     const uint64_t end = start + command->slice.len;
     uint64_t last_offset = start;
-    bool last_is_hole = false;
+    bool last_is_zero = false;
     uint64_t i;
     struct command *newcommand;
-    int dummy = 0;
 
     /* Iterate over whole blocks in the command, starting on a block
      * boundary.
@@ -409,73 +530,75 @@ finished_read (void *vp, int *error)
          i + sparse_size <= end;
          i += sparse_size) {
       if (is_zero (slice_ptr (command->slice) + i-start, sparse_size)) {
-        /* It's a hole.  If the last was a hole too then we do nothing
-         * here which coalesces.  Otherwise write the last data and
-         * start a new hole.
+        /* It's a zero range.  If the last was a zero too then we do
+         * nothing here which coalesces.  Otherwise write the last data
+         * and start a new zero range.
          */
-        if (!last_is_hole) {
+        if (!last_is_zero) {
           /* Write the last data (if any). */
           if (i - last_offset > 0) {
-            newcommand = copy_subcommand (command,
-                                          last_offset, i - last_offset,
-                                          false);
+            newcommand = create_subcommand (command,
+                                            last_offset, i - last_offset,
+                                            false);
             dst->ops->asynch_write (dst, newcommand,
                                     (nbd_completion_callback) {
-                                      .callback = free_command,
+                                      .callback = finished_command,
                                       .user_data = newcommand,
                                     });
           }
-          /* Start the new hole. */
+          /* Start the new zero range. */
           last_offset = i;
-          last_is_hole = true;
+          last_is_zero = true;
         }
       }
       else {
         /* It's data.  If the last was data too, do nothing =>
-         * coalesce.  Otherwise write the last hole and start a new
-         * data.
+         * coalesce.  Otherwise write the last zero range and start a
+         * new data.
          */
-        if (last_is_hole) {
-          /* Write the last hole (if any). */
+        if (last_is_zero) {
+          /* Write the last zero range (if any). */
           if (i - last_offset > 0) {
-            newcommand = copy_subcommand (command,
-                                          last_offset, i - last_offset,
-                                          true);
+            newcommand = create_subcommand (command,
+                                            last_offset, i - last_offset,
+                                            true);
+            decrease_queue_size (command->worker, newcommand->slice.len);
             fill_dst_range_with_zeroes (newcommand);
           }
           /* Start the new data. */
           last_offset = i;
-          last_is_hole = false;
+          last_is_zero = false;
         }
       }
     } /* for i */
 
     /* Write the last_offset up to i. */
     if (i - last_offset > 0) {
-      if (!last_is_hole) {
-        newcommand = copy_subcommand (command,
-                                      last_offset, i - last_offset,
-                                      false);
+      if (!last_is_zero) {
+        newcommand = create_subcommand (command,
+                                        last_offset, i - last_offset,
+                                        false);
         dst->ops->asynch_write (dst, newcommand,
                                 (nbd_completion_callback) {
-                                  .callback = free_command,
+                                  .callback = finished_command,
                                   .user_data = newcommand,
                                 });
       }
       else {
-        newcommand = copy_subcommand (command,
-                                      last_offset, i - last_offset,
-                                      true);
+        newcommand = create_subcommand (command,
+                                        last_offset, i - last_offset,
+                                        true);
+        decrease_queue_size (command->worker, newcommand->slice.len);
         fill_dst_range_with_zeroes (newcommand);
       }
     }
 
     /* There may be an unaligned tail, so write that. */
     if (end - i > 0) {
-      newcommand = copy_subcommand (command, i, end - i, false);
+      newcommand = create_subcommand (command, i, end - i, false);
       dst->ops->asynch_write (dst, newcommand,
                               (nbd_completion_callback) {
-                                .callback = free_command,
+                                .callback = finished_command,
                                 .user_data = newcommand,
                               });
     }
@@ -483,15 +606,15 @@ finished_read (void *vp, int *error)
     /* Free the original command since it has been split into
      * subcommands and the original is no longer needed.
      */
-    free_command (command, &dummy);
+    free_command (command);
   }
 
   return 1; /* auto-retires the command */
 }
 
 /* Fill a range in dst with zeroes.  This is called from the copying
- * loop when we see a hole in the source.  Depending on the command
- * line flags this could mean:
+ * loop when we see a zero range in the source.  Depending on the
+ * command line flags this could mean:
  *
  * --destination-is-zero:
  *                 do nothing
@@ -510,7 +633,6 @@ fill_dst_range_with_zeroes (struct command *command)
 {
   char *data;
   size_t data_size;
-  int dummy = 0;
 
   if (destination_is_zero)
     goto free_and_return;
@@ -518,7 +640,7 @@ fill_dst_range_with_zeroes (struct command *command)
   /* Try efficient zeroing. */
   if (dst->ops->asynch_zero (dst, command,
                              (nbd_completion_callback) {
-                               .callback = free_command,
+                               .callback = finished_command,
                                .user_data = command,
                              },
                              allocated))
@@ -546,21 +668,36 @@ fill_dst_range_with_zeroes (struct command *command)
   free (data);
 
  free_and_return:
-  free_command (command, &dummy);
+  free_command (command);
 }
 
 static int
-free_command (void *vp, int *error)
+finished_command (void *vp, int *error)
 {
   struct command *command = vp;
-  struct buffer *buffer = command->slice.buffer;
 
   if (*error) {
-    fprintf (stderr, "write at offset %" PRId64 " failed: %s\n",
-             command->offset, strerror (*error));
+    fprintf (stderr, "%s: write at offset %" PRId64 " failed: %s\n",
+             prog, command->offset, strerror (*error));
     exit (EXIT_FAILURE);
   }
 
+  if (command->slice.buffer)
+    decrease_queue_size (command->worker, command->slice.len);
+
+  free_command (command);
+
+  return 1; /* auto-retires the command */
+}
+
+static void
+free_command (struct command *command)
+{
+  if (command == NULL)
+    return;
+
+  struct buffer *buffer = command->slice.buffer;
+
   if (buffer != NULL) {
     if (--buffer->refs == 0) {
       free (buffer->data);
@@ -569,6 +706,4 @@ free_command (void *vp, int *error)
   }
 
   free (command);
-
-  return 1; /* auto-retires the command */
 }
