@@ -27,10 +27,13 @@
 
 #include "nbdcopy.h"
 
+#include "const-string-vector.h"
+#include "ispowerof2.h"
+#include "vector.h"
+
 static struct rw_ops nbd_ops;
 
 DEFINE_VECTOR_TYPE (handles, struct nbd_handle *)
-DEFINE_VECTOR_TYPE (const_string_vector, const char *);
 
 struct rw_nbd {
   struct rw rw;
@@ -61,6 +64,25 @@ open_one_nbd_handle (struct rw_nbd *rwn)
 
   nbd_set_debug (nbd, verbose);
 
+  /* Set the handle name for debugging.  We could use rwn->rw.name
+   * here but it is usually set to the lengthy NBD URI
+   * (eg. "nbd://localhost:10809") which makes debug messages very
+   * long.
+   */
+  if (verbose) {
+    char *name;
+    const size_t index = rwn->handles.len;
+
+    if (asprintf (&name, "%s%zu",
+                  rwn->d == READING ? "src" : "dst",
+                  index) == -1) {
+      perror ("asprintf");
+      exit (EXIT_FAILURE);
+    }
+    nbd_set_handle_name (nbd, name);
+    free (name);
+  }
+
   if (extents && rwn->d == READING &&
       nbd_add_meta_context (nbd, "base:allocation") == -1) {
     fprintf (stderr, "%s: %s\n", prog, nbd_get_error ());
@@ -90,13 +112,26 @@ open_one_nbd_handle (struct rw_nbd *rwn)
   /* Cache these.  We assume with multi-conn that each handle will act
    * the same way.
    */
-  if (rwn->handles.size == 0) {
+  if (rwn->handles.len == 0) {
+    int64_t block_size;
+
     rwn->can_zero = nbd_can_zero (nbd) > 0;
+
     rwn->rw.size = nbd_get_size (nbd);
     if (rwn->rw.size == -1) {
       fprintf (stderr, "%s: %s: %s\n", prog, rwn->rw.name, nbd_get_error ());
       exit (EXIT_FAILURE);
     }
+
+    block_size = nbd_get_block_size (nbd, LIBNBD_SIZE_PREFERRED);
+    if (block_size == -1) {
+      fprintf (stderr, "%s: %s: %s\n", prog, rwn->rw.name, nbd_get_error ());
+      exit (EXIT_FAILURE);
+    }
+    if (block_size > 0 && is_power_of_2 (block_size))
+      rwn->rw.preferred = block_size;
+    else
+      rwn->rw.preferred = 4096;
   }
 
   if (handles_append (&rwn->handles, nbd) == -1) {
@@ -157,7 +192,7 @@ nbd_ops_close (struct rw *rw)
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
   size_t i;
 
-  for (i = 0; i < rwn->handles.size; ++i) {
+  for (i = 0; i < rwn->handles.len; ++i) {
     if (nbd_shutdown (rwn->handles.ptr[i], 0) == -1) {
       fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
       exit (EXIT_FAILURE);
@@ -176,7 +211,7 @@ nbd_ops_flush (struct rw *rw)
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
   size_t i;
 
-  for (i = 0; i < rwn->handles.size; ++i) {
+  for (i = 0; i < rwn->handles.len; ++i) {
     if (nbd_flush (rwn->handles.ptr[i], 0) == -1) {
       fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
       exit (EXIT_FAILURE);
@@ -189,7 +224,7 @@ nbd_ops_is_read_only (struct rw *rw)
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
-  if (rwn->handles.size > 0)
+  if (rwn->handles.len > 0)
     return nbd_is_read_only (rwn->handles.ptr[0]);
   else
     return false;
@@ -200,7 +235,7 @@ nbd_ops_can_extents (struct rw *rw)
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
-  if (rwn->handles.size > 0)
+  if (rwn->handles.len > 0)
     return nbd_can_meta_context (rwn->handles.ptr[0], "base:allocation");
   else
     return false;
@@ -211,7 +246,7 @@ nbd_ops_can_multi_conn (struct rw *rw)
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
-  if (rwn->handles.size > 0)
+  if (rwn->handles.len > 0)
     return nbd_can_multi_conn (rwn->handles.ptr[0]);
   else
     return false;
@@ -226,12 +261,12 @@ nbd_ops_start_multi_conn (struct rw *rw)
   for (i = 1; i < connections; ++i)
     open_one_nbd_handle (rwn);
 
-  assert (rwn->handles.size == connections);
+  assert (rwn->handles.len == connections);
 }
 
 static size_t
 nbd_ops_synch_read (struct rw *rw,
-                void *data, size_t len, uint64_t offset)
+                    void *data, size_t len, uint64_t offset)
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
@@ -250,7 +285,7 @@ nbd_ops_synch_read (struct rw *rw,
 
 static void
 nbd_ops_synch_write (struct rw *rw,
-                 const void *data, size_t len, uint64_t offset)
+                     const void *data, size_t len, uint64_t offset)
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
@@ -284,7 +319,7 @@ nbd_ops_asynch_read (struct rw *rw,
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
-  if (nbd_aio_pread (rwn->handles.ptr[command->index],
+  if (nbd_aio_pread (rwn->handles.ptr[command->worker->index],
                      slice_ptr (command->slice),
                      command->slice.len, command->offset,
                      cb, 0) == -1) {
@@ -300,7 +335,7 @@ nbd_ops_asynch_write (struct rw *rw,
 {
   struct rw_nbd *rwn = (struct rw_nbd *) rw;
 
-  if (nbd_aio_pwrite (rwn->handles.ptr[command->index],
+  if (nbd_aio_pwrite (rwn->handles.ptr[command->worker->index],
                       slice_ptr (command->slice),
                       command->slice.len, command->offset,
                       cb, 0) == -1) {
@@ -320,7 +355,7 @@ nbd_ops_asynch_zero (struct rw *rw, struct command *command,
 
   assert (command->slice.len <= UINT32_MAX);
 
-  if (nbd_aio_zero (rwn->handles.ptr[command->index],
+  if (nbd_aio_zero (rwn->handles.ptr[command->worker->index],
                     command->slice.len, command->offset,
                     cb, allocate ? LIBNBD_CMD_FLAG_NO_HOLE : 0) == -1) {
     fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
@@ -329,6 +364,135 @@ nbd_ops_asynch_zero (struct rw *rw, struct command *command,
   return true;
 }
 
+static unsigned
+nbd_ops_in_flight (struct rw *rw, size_t index)
+{
+  struct rw_nbd *rwn = (struct rw_nbd *) rw;
+
+  /* Since the commands are auto-retired in the callbacks we don't
+   * need to count "done" commands.
+   */
+  return nbd_aio_in_flight (rwn->handles.ptr[index]);
+}
+
+static void
+nbd_ops_get_polling_fd (struct rw *rw, size_t index,
+                        int *fd, int *direction)
+{
+  struct rw_nbd *rwn = (struct rw_nbd *) rw;
+  struct nbd_handle *nbd;
+
+  nbd = rwn->handles.ptr[index];
+
+  *fd = nbd_aio_get_fd (nbd);
+  if (*fd == -1)
+    goto error;
+
+  *direction = nbd_aio_get_direction (nbd);
+  if (*direction == -1)
+    goto error;
+
+  return;
+
+error:
+  fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
+  exit (EXIT_FAILURE);
+}
+
+static void
+nbd_ops_asynch_notify_read (struct rw *rw, size_t index)
+{
+  struct rw_nbd *rwn = (struct rw_nbd *) rw;
+  if (nbd_aio_notify_read (rwn->handles.ptr[index]) == -1) {
+    fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
+    exit (EXIT_FAILURE);
+  }
+}
+
+static void
+nbd_ops_asynch_notify_write (struct rw *rw, size_t index)
+{
+  struct rw_nbd *rwn = (struct rw_nbd *) rw;
+  if (nbd_aio_notify_write (rwn->handles.ptr[index]) == -1) {
+    fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
+    exit (EXIT_FAILURE);
+  }
+}
+
+/* Get the extents.
+ *
+ * This is done synchronously, but that's fine because commands from
+ * the previous work range in flight continue to run, it's difficult
+ * to (sanely) start new work until we have the full list of extents,
+ * and in almost every case the remote NBD server can answer our
+ * request for extents in a single round trip.
+ */
+static int add_extent (void *vp, const char *metacontext,
+                       uint64_t offset, uint32_t *entries, size_t nr_entries,
+                       int *error);
+
+static void
+nbd_ops_get_extents (struct rw *rw, size_t index,
+                     uint64_t offset, uint64_t count,
+                     extent_list *ret)
+{
+  struct rw_nbd *rwn = (struct rw_nbd *) rw;
+  extent_list exts = empty_vector;
+  struct nbd_handle *nbd;
+
+  nbd = rwn->handles.ptr[index];
+
+  ret->len = 0;
+
+  while (count > 0) {
+    const uint64_t old_offset = offset;
+    size_t i;
+
+    exts.len = 0;
+    if (nbd_block_status (nbd, count, offset,
+                          (nbd_extent_callback) {
+                            .user_data = &exts,
+                            .callback = add_extent
+                          }, 0) == -1) {
+      /* XXX We could call default_get_extents, but unclear if it's
+       * the right thing to do if the server is returning errors.
+       */
+      fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
+      exit (EXIT_FAILURE);
+    }
+
+    /* Copy the extents returned into the final list (ret). */
+    for (i = 0; i < exts.len; ++i) {
+      uint64_t d;
+
+      assert (exts.ptr[i].offset == offset);
+      if (exts.ptr[i].offset + exts.ptr[i].length > offset + count) {
+        d = exts.ptr[i].offset + exts.ptr[i].length - offset - count;
+        exts.ptr[i].length -= d;
+        assert (exts.ptr[i].offset + exts.ptr[i].length == offset + count);
+      }
+      if (exts.ptr[i].length == 0)
+        continue;
+      if (extent_list_append (ret, exts.ptr[i]) == -1) {
+        perror ("realloc");
+        exit (EXIT_FAILURE);
+      }
+
+      offset += exts.ptr[i].length;
+      count -= exts.ptr[i].length;
+    }
+
+    /* The server should always make progress. */
+    if (offset == old_offset) {
+      fprintf (stderr, "%s: NBD server is broken: it is not returning extent information.\nTry nbdcopy --no-extents as a workaround.\n",
+               rw->name);
+      exit (EXIT_FAILURE);
+    }
+  }
+
+  free (exts.ptr);
+}
+
 static int
 add_extent (void *vp, const char *metacontext,
             uint64_t offset, uint32_t *entries, size_t nr_entries,
@@ -364,129 +528,6 @@ add_extent (void *vp, const char *metacontext,
   return 0;
 }
 
-static unsigned
-nbd_ops_in_flight (struct rw *rw, uintptr_t index)
-{
-  struct rw_nbd *rwn = (struct rw_nbd *) rw;
-
-  /* Since the commands are auto-retired in the callbacks we don't
-   * need to count "done" commands.
-   */
-  return nbd_aio_in_flight (rwn->handles.ptr[index]);
-}
-
-static void
-nbd_ops_get_polling_fd (struct rw *rw, uintptr_t index,
-                        int *fd, int *direction)
-{
-  struct rw_nbd *rwn = (struct rw_nbd *) rw;
-  struct nbd_handle *nbd;
-
-  nbd = rwn->handles.ptr[index];
-
-  *fd = nbd_aio_get_fd (nbd);
-  if (*fd == -1)
-    goto error;
-
-  *direction = nbd_aio_get_direction (nbd);
-  if (*direction == -1)
-    goto error;
-
-  return;
-
-error:
-  fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
-  exit (EXIT_FAILURE);
-}
-
-static void
-nbd_ops_asynch_notify_read (struct rw *rw, uintptr_t index)
-{
-  struct rw_nbd *rwn = (struct rw_nbd *) rw;
-  if (nbd_aio_notify_read (rwn->handles.ptr[index]) == -1) {
-    fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
-    exit (EXIT_FAILURE);
-  }
-}
-
-static void
-nbd_ops_asynch_notify_write (struct rw *rw, uintptr_t index)
-{
-  struct rw_nbd *rwn = (struct rw_nbd *) rw;
-  if (nbd_aio_notify_write (rwn->handles.ptr[index]) == -1) {
-    fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
-    exit (EXIT_FAILURE);
-  }
-}
-
-/* This is done synchronously, but that's fine because commands from
- * the previous work range in flight continue to run, it's difficult
- * to (sanely) start new work until we have the full list of extents,
- * and in almost every case the remote NBD server can answer our
- * request for extents in a single round trip.
- */
-static void
-nbd_ops_get_extents (struct rw *rw, uintptr_t index,
-                     uint64_t offset, uint64_t count,
-                     extent_list *ret)
-{
-  struct rw_nbd *rwn = (struct rw_nbd *) rw;
-  extent_list exts = empty_vector;
-  struct nbd_handle *nbd;
-
-  nbd = rwn->handles.ptr[index];
-
-  ret->size = 0;
-
-  while (count > 0) {
-    const uint64_t old_offset = offset;
-    size_t i;
-
-    exts.size = 0;
-    if (nbd_block_status (nbd, count, offset,
-                          (nbd_extent_callback) {
-                            .user_data = &exts,
-                            .callback = add_extent
-                          }, 0) == -1) {
-      /* XXX We could call default_get_extents, but unclear if it's
-       * the right thing to do if the server is returning errors.
-       */
-      fprintf (stderr, "%s: %s\n", rw->name, nbd_get_error ());
-      exit (EXIT_FAILURE);
-    }
-
-    /* Copy the extents returned into the final list (ret). */
-    for (i = 0; i < exts.size; ++i) {
-      uint64_t d;
-
-      assert (exts.ptr[i].offset == offset);
-      if (exts.ptr[i].offset + exts.ptr[i].length > offset + count) {
-        d = exts.ptr[i].offset + exts.ptr[i].length - offset - count;
-        exts.ptr[i].length -= d;
-        assert (exts.ptr[i].offset + exts.ptr[i].length == offset + count);
-      }
-      if (exts.ptr[i].length == 0)
-        continue;
-      if (extent_list_append (ret, exts.ptr[i]) == -1) {
-        perror ("realloc");
-        exit (EXIT_FAILURE);
-      }
-
-      offset += exts.ptr[i].length;
-      count -= exts.ptr[i].length;
-    }
-
-    /* The server should always make progress. */
-    if (offset == old_offset) {
-      fprintf (stderr, "%s: NBD server is broken: it is not returning extent information.\nTry nbdcopy --no-extents as a workaround.\n",
-               rw->name);
-      exit (EXIT_FAILURE);
-    }
-  }
-
-  free (exts.ptr);
-}
-
 static struct rw_ops nbd_ops = {
   .ops_name = "nbd_ops",
   .close = nbd_ops_close,
