@@ -81,28 +81,56 @@ let type_of_ret =
   | RUIntPtr -> "uintptr_t"
   | RFlags _ -> "uint32_t"
 
-let rec name_of_arg = function
-| Bool n -> [n]
-| BytesIn (n, len) -> [n; len]
-| BytesOut (n, len) -> [n; len]
-| BytesPersistIn (n, len) -> [n; len]
-| BytesPersistOut (n, len) -> [n; len]
-| Closure { cbname } ->
-   [ sprintf "%s_callback" cbname; sprintf "%s_user_data" cbname ]
-| Enum (n, _) -> [n]
-| Fd n -> [n]
-| Flags (n, _) -> [n]
-| Int n -> [n]
-| Int64 n -> [n]
-| Path n -> [n]
-| SizeT n -> [n]
-| SockAddrAndLen (n, len) -> [n; len]
-| String n -> [n]
-| StringList n -> [n]
-| UInt n -> [n]
-| UInt32 n -> [n]
-| UInt64 n -> [n]
-| UIntPtr n -> [n]
+let name_of_arg =
+  function
+  | Bool n -> [n]
+  | BytesIn (n, len) -> [n; len]
+  | BytesOut (n, len) -> [n; len]
+  | BytesPersistIn (n, len) -> [n; len]
+  | BytesPersistOut (n, len) -> [n; len]
+  | Closure { cbname } ->
+     [ sprintf "%s_callback" cbname; sprintf "%s_user_data" cbname ]
+  | Enum (n, _) -> [n]
+  | Fd n -> [n]
+  | Flags (n, _) -> [n]
+  | Int n -> [n]
+  | Int64 n -> [n]
+  | Path n -> [n]
+  | SizeT n -> [n]
+  | SockAddrAndLen (n, len) -> [n; len]
+  | String n -> [n]
+  | StringList n -> [n]
+  | UInt n -> [n]
+  | UInt32 n -> [n]
+  | UInt64 n -> [n]
+  | UIntPtr n -> [n]
+
+let name_of_optarg =
+  function
+  | OClosure { cbname } -> cbname
+  | OFlags (n, _, _) -> n
+
+(* Map function arguments to __attribute__((nonnull)) annotations.
+ * Because a single arg may map to multiple C parameters this
+ * returns a list.  Note: true => add attribute nonnull.
+ *)
+let arg_attr_nonnull =
+  function
+  (* BytesIn/Out are passed using a non-null pointer, and size_t *)
+  | BytesIn _ | BytesOut _
+  | BytesPersistIn _ | BytesPersistOut _ -> [ true; false ]
+  (* sockaddr is also non-null pointer, and length *)
+  | SockAddrAndLen (n, len) -> [ true; false ]
+  (* strings should be marked as non-null *)
+  | Path _ | String _ -> [ true ]
+  (* list of strings should be marked as non-null *)
+  | StringList n -> [ true ]
+  (* numeric and other non-pointer types are not able to be null *)
+  | Bool _ | Closure _ | Enum _ | Fd _ | Flags _
+  | Int _ | Int64 _ | SizeT _
+  | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> [ false ]
+
+let optarg_attr_nonnull (OClosure _ | OFlags _) = [ false ]
 
 let rec print_arg_list ?(wrap = false) ?maxcol ?handle ?types ?(parens = true)
           ?closure_style args optargs =
@@ -210,10 +238,34 @@ let print_call ?wrap ?maxcol ?closure_style name args optargs ret =
   pr "%s nbd_%s " (type_of_ret ret) name;
   print_arg_list ~handle:true ?wrap ?maxcol ?closure_style args optargs
 
-let print_extern ?wrap ?closure_style name args optargs ret =
+let print_fndecl ?wrap ?closure_style name args optargs ret =
   pr "extern ";
   print_call ?wrap ?closure_style name args optargs ret;
-  pr ";\n"
+
+  (* For RString, note that the caller must free() the argument.
+   * Since this is used in a public header, we must use gcc's spelling
+   * of __builtin_free in case bare free is defined as a macro.
+   *)
+  (match ret with
+   | RString ->
+      pr "\n    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(__builtin_free)";
+   | _ -> ()
+  );
+
+  (* Output __attribute__((nonnull)) for the function parameters:
+   * eg. struct nbd_handle *, int, char *
+   *     => [ true, false, true ]
+   *     => LIBNBD_ATTRIBUTE_NONNULL(1,3)
+   *     => __attribute__((nonnull(1,3)))
+   *)
+  let nns : bool list =
+    [ true ] (* struct nbd_handle * *)
+    @ List.flatten (List.map arg_attr_nonnull args)
+    @ List.flatten (List.map optarg_attr_nonnull optargs) in
+  let nns = List.mapi (fun i b -> (i+1, b)) nns in
+  let nns = filter_map (fun (i, b) -> if b then Some i else None) nns in
+  let nns : string list = List.map string_of_int nns in
+  pr "\n    LIBNBD_ATTRIBUTE_NONNULL(%s);\n" (String.concat "," nns)
 
 let rec print_cbarg_list ?(wrap = false) ?maxcol ?types ?(parens = true)
           cbargs =
@@ -300,9 +352,9 @@ let print_closure_structs () =
   ) oclosures;
   pr "\n"
 
-let print_extern_and_define ?wrap name args optargs ret =
+let print_fndecl_and_define ?wrap name args optargs ret =
   let name_upper = String.uppercase_ascii name in
-  print_extern ?wrap name args optargs ret;
+  print_fndecl ?wrap name args optargs ret;
   pr "#define LIBNBD_HAVE_NBD_%s 1\n" name_upper;
   pr "\n"
 
@@ -346,6 +398,26 @@ let generate_include_libnbd_h () =
   pr "extern \"C\" {\n";
   pr "#endif\n";
   pr "\n";
+  pr "#if defined(__GNUC__)\n";
+  pr "#define LIBNBD_GCC_VERSION \\\n";
+  pr "    (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)\n";
+  pr "#endif\n";
+  pr "\n";
+  pr "#ifndef LIBNBD_ATTRIBUTE_NONNULL\n";
+  pr "#if defined(__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
+  pr "#define LIBNBD_ATTRIBUTE_NONNULL(...) __attribute__((__nonnull__(__VA_ARGS__)))\n";
+  pr "#else\n";
+  pr "#define LIBNBD_ATTRIBUTE_NONNULL(...)\n";
+  pr "#endif\n";
+  pr "#endif /* ! defined LIBNBD_ATTRIBUTE_NONNULL */\n";
+  pr "\n";
+  pr "#if defined(__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
+  pr "#define LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(fn) \\\n";
+  pr "  __attribute__((__malloc__, __malloc__(fn, 1)))\n";
+  pr "#else\n";
+  pr "#define LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(fn)\n";
+  pr "#endif\n";
+  pr "\n";
   pr "struct nbd_handle;\n";
   pr "\n";
   List.iter (
@@ -363,11 +435,11 @@ let generate_include_libnbd_h () =
       List.iter (
         fun (flag, i) ->
           let flag = sprintf "LIBNBD_%s_%s" flag_prefix flag in
-          pr "#define %-40s 0x%02x\n" flag i;
+          pr "#define %-40s 0x%02xU\n" flag i;
           mask := !mask lor i
       ) flags;
       let flag = sprintf "LIBNBD_%s_MASK" flag_prefix in
-      pr "#define %-40s 0x%02x\n" flag !mask;
+      pr "#define %-40s 0x%02xU\n" flag !mask;
       pr "\n"
   ) all_flags;
   List.iter (
@@ -376,12 +448,13 @@ let generate_include_libnbd_h () =
       pr "#define %-40s %d\n" n i
   ) constants;
   pr "\n";
-  pr "extern struct nbd_handle *nbd_create (void);\n";
-  pr "#define LIBNBD_HAVE_NBD_CREATE 1\n";
-  pr "\n";
-  pr "extern void nbd_close (struct nbd_handle *h);\n";
+  pr "extern void nbd_close (struct nbd_handle *h); /* h can be NULL */\n";
   pr "#define LIBNBD_HAVE_NBD_CLOSE 1\n";
   pr "\n";
+  pr "extern struct nbd_handle *nbd_create (void)\n";
+  pr "    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(nbd_close);\n";
+  pr "#define LIBNBD_HAVE_NBD_CREATE 1\n";
+  pr "\n";
   pr "extern const char *nbd_get_error (void);\n";
   pr "#define LIBNBD_HAVE_NBD_GET_ERROR 1\n";
   pr "\n";
@@ -391,7 +464,7 @@ let generate_include_libnbd_h () =
   print_closure_structs ();
   List.iter (
     fun (name, { args; optargs; ret }) ->
-      print_extern_and_define ~wrap:true name args optargs ret
+      print_fndecl_and_define ~wrap:true name args optargs ret
   ) handle_calls;
   List.iter (
     fun (ns, ctxts) -> print_ns ns ctxts
@@ -410,7 +483,7 @@ let generate_lib_unlocked_h () =
   pr "\n";
   List.iter (
     fun (name, { args; optargs; ret }) ->
-    print_extern ~wrap:true ~closure_style:Pointer ("unlocked_" ^ name)
+    print_fndecl ~wrap:true ~closure_style:Pointer ("unlocked_" ^ name)
       args optargs ret
   ) handle_calls;
   pr "\n";
@@ -486,17 +559,16 @@ let generate_lib_api_c () =
     (* Lock the handle. *)
     if is_locked then
       pr "  pthread_mutex_lock (&h->lock);\n";
-    if may_set_error then (
-      print_trace_enter args optargs;
-      pr "\n"
-    );
+    print_trace_enter name args optargs may_set_error;
+    pr "\n";
 
     (* Sanitize read buffers before any error is possible. *)
     List.iter (
       function
       | BytesOut (n, count)
       | BytesPersistOut (n, count) ->
-         pr "  memset (%s, 0, %s);\n" n count
+         pr "  if (h->pread_initialize)\n";
+         pr "    memset (%s, 0, %s);\n" n count
       | _ -> ()
     ) args;
 
@@ -569,7 +641,12 @@ let generate_lib_api_c () =
          need_out_label := true
       | Flags (n, flags) ->
          print_flags_check n flags None
-      | String n ->
+      | BytesIn (n, _) | BytesOut (n, _)
+      | BytesPersistIn (n, _) | BytesPersistOut (n, _)
+      | SockAddrAndLen (n, _)
+      | Path n
+      | String n
+      | StringList n ->
          let value = match errcode with
            | Some value -> value
            | None -> assert false in
@@ -596,10 +673,8 @@ let generate_lib_api_c () =
     pr "\n";
     if !need_out_label then
       pr " out:\n";
-    if may_set_error then (
-      print_trace_leave ret;
-      pr "\n"
-    );
+    print_trace_leave name ret may_set_error;
+    pr "\n";
     (* Finish any closures not transferred to state machine. *)
     List.iter (
       function
@@ -623,7 +698,7 @@ let generate_lib_api_c () =
     pr "\n"
 
   (* Print the trace when we enter a call with debugging enabled. *)
-  and print_trace_enter args optargs =
+  and print_trace_enter name args optargs may_set_error =
     pr "  if_debug (h) {\n";
     List.iter (
       function
@@ -643,7 +718,11 @@ let generate_lib_api_c () =
       | Int64 _ | SizeT _
       | SockAddrAndLen _ | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> ()
     ) args;
-    pr "    debug (h, \"enter:";
+    if may_set_error then
+      pr "    debug (h, \"enter:"
+    else (
+      pr "    debug_direct (h, \"nbd_%s\", \"enter:" name
+    );
     List.iter (
       function
       | Bool n -> pr " %s=%%s" n
@@ -712,43 +791,44 @@ let generate_lib_api_c () =
     ) args;
     pr "  }\n"
   (* Print the trace when we leave a call with debugging enabled. *)
-  and print_trace_leave ret =
+  and print_trace_leave name ret may_set_error =
     pr "  if_debug (h) {\n";
-    let errcode = errcode_of_ret ret in
-    (match errcode with
-     | Some r ->
-        pr "    if (ret == %s)\n" r;
-     | None ->
-        pr "    if (0)\n";
+    if may_set_error then (
+      (match errcode_of_ret ret with
+       | Some r ->
+          pr "    if (ret == %s)\n" r;
+       | None -> assert false
+      );
+      pr "      debug (h, \"leave: error=\\\"%%s\\\"\", nbd_get_error ());\n";
+      pr "    else {\n";
+      (match ret with
+       | RString ->
+          pr "      char *ret_printable =\n";
+          pr "          nbd_internal_printable_string (ret);\n"
+       | _ -> ()
+      );
+      pr "      debug (h, \"leave: ret=\" "
+    )
+    else (
+      pr "    debug_direct (h, \"nbd_%s\", \"leave: ret=\" " name
     );
-    pr "      debug (h, \"leave: error=\\\"%%s\\\"\", nbd_get_error ());\n";
-    pr "    else {\n";
-    (match ret with
-     | RStaticString | RString ->
-        pr "      char *ret_printable =\n";
-        pr "          nbd_internal_printable_string (ret);\n"
-     | RBool | RErr | RFd | RInt
-     | RInt64 | RCookie | RSizeT
-     | RUInt | RUIntPtr | REnum _ | RFlags _ -> ()
-    );
-    pr "      debug (h, \"leave: ret=\" ";
     (match ret with
      | RBool | RErr | RFd | RInt | REnum _ -> pr "\"%%d\", ret"
      | RInt64 | RCookie -> pr "\"%%\" PRIi64 \"\", ret"
      | RSizeT -> pr "\"%%zd\", ret"
-     | RStaticString | RString ->
-        pr "\"%%s\", ret_printable ? ret_printable : \"\""
+     | RString -> pr "\"%%s\", ret_printable ? ret_printable : \"\""
+     | RStaticString -> pr "\"%%s\", ret"
      | RUInt | RFlags _ -> pr "\"%%u\", ret"
      | RUIntPtr -> pr "\"%%\" PRIuPTR, ret"
     );
     pr ");\n";
-    (match ret with
-     | RStaticString | RString -> pr "      free (ret_printable);\n"
-     | RBool | RErr | RFd | RInt
-     | RInt64 | RCookie | RSizeT
-     | RUInt | REnum _ | RFlags _ | RUIntPtr -> ()
+    if may_set_error then (
+      (match ret with
+       | RString -> pr "      free (ret_printable);\n"
+       | _ -> ()
+      );
+      pr "    }\n";
     );
-    pr "    }\n";
     pr "  }\n"
   in
 
@@ -764,6 +844,12 @@ let generate_lib_api_c () =
   pr "\n";
   pr "#include <pthread.h>\n";
   pr "\n";
+  pr "/* GCC will remove NULL checks from this file for any parameter\n";
+  pr " * annotated with attribute((nonnull)).  See RHBZ#1041336.  To\n";
+  pr " * avoid this, disable the attribute when including libnbd.h.\n";
+  pr " */\n";
+  pr "#define LIBNBD_ATTRIBUTE_NONNULL(...)\n";
+  pr "\n";
   pr "#include \"libnbd.h\"\n";
   pr "#include \"internal.h\"\n";
   pr "\n";
@@ -835,7 +921,14 @@ let generate_docs_nbd_pod name { args; optargs; ret;
                                  first_version = (major, minor) } () =
   pr "=head1 NAME\n";
   pr "\n";
-  pr_wrap ' ' (fun () -> pr "nbd_%s - %s" name shortdesc);
+  pr "nbd_%s -" name;
+  (* Don't use pr_wrap here because it leaves trailing whitespace. *)
+  let words = nsplit " " shortdesc in
+  List.iter (
+    fun word ->
+      if output_column () + String.length word > 70 then pr "\n%s" word
+      else pr " %s" word
+  ) words;
   pr "\n";
   pr "\n";
 
@@ -905,6 +998,7 @@ let generate_docs_nbd_pod name { args; optargs; ret;
 
   pr "=head1 ERRORS\n";
   pr "\n";
+
   if may_set_error then (
     let value = match errcode with
       | Some value -> value
@@ -918,6 +1012,21 @@ let generate_docs_nbd_pod name { args; optargs; ret;
     pr "This function does not fail.\n";
   pr "\n";
 
+  pr "The following parameters must not be NULL: C<h>";
+  List.iter (
+    fun arg ->
+      let nns = arg_attr_nonnull arg in
+      if List.mem true nns then pr ", C<%s>" (List.hd (name_of_arg arg))
+  ) args;
+  List.iter (
+    fun arg ->
+      let nns = optarg_attr_nonnull arg in
+      if List.mem true nns then pr ", C<%s>" (name_of_optarg arg)
+  ) optargs;
+  pr ".\n";
+  pr "For more information see L<libnbd(3)/Non-NULL parameters>.\n";
+  pr "\n";
+
   if permitted_states <> [] then (
     pr "=head1 HANDLE STATE\n";
     pr "\n";
@@ -949,9 +1058,10 @@ let generate_docs_nbd_pod name { args; optargs; ret;
       (try
          while true do
            let line = input_line chan in
-           if String.length line > 60 then
+           let len = String.length line in
+           if len > 60 then
              failwithf "%s: %s: line too long in example" name filename;
-           pr " %s\n" line
+           if len = 0 then pr "\n" else pr " %s\n" line
          done
        with End_of_file -> ()
       );
