@@ -49,7 +49,7 @@ and ocaml_arg_to_string = function
   | Int _ -> "int"
   | Int64 _ -> "int64"
   | Path _ -> "string"
-  | SockAddrAndLen _ -> "string" (* XXX not impl *)
+  | SockAddrAndLen _ -> "Unix.sockaddr"
   | SizeT _ -> "int" (* OCaml int type is always sufficient for counting *)
   | String _ -> "string"
   | StringList _ -> "string list"
@@ -134,10 +134,10 @@ let generate_ocaml_nbd_mli () =
     [ocaml/examples] subdirectory.
 *)
 
-exception Error of string * int
+exception Error of string * Unix.error option
 (** Exception thrown when an API call fails.
 
-    The string is the error message, and the int is the raw errno
+    The string is the error message, and the int is the {!Unix.error}
     (if available).
 *)
 
@@ -211,6 +211,10 @@ module Buffer : sig
 end
 (** Persistent buffer used in AIO calls. *)
 
+val errno_of_unix_error : Unix.error -> int
+(** Return the raw C errno corresponding to a {!Unix.error}.  This
+    can be used in callbacks to update the [int ref] parameter. *)
+
 type t
 (** The handle. *)
 
@@ -243,7 +247,12 @@ val with_handle : (t -> 'a) -> 'a
     fun (name, { args; optargs; ret; shortdesc; longdesc }) ->
       pr "val %s : %s\n" name (ocaml_fundecl_to_string args optargs ret);
 
-      pr "(** %s\n" shortdesc;
+      pr "(** [NBD.%s t" name;
+      List.iter (fun arg -> arg |> ocaml_name_of_optarg |> pr " ?%s") optargs;
+      List.iter (fun arg -> arg |> ocaml_name_of_arg |> pr " %s") args;
+      pr "]\n";
+      pr "\n";
+      pr "    %s\n" shortdesc;
       pr "\n";
       pr "%s" (String.concat "\n" (pod2text longdesc));
       pr "*)\n";
@@ -255,13 +264,13 @@ let generate_ocaml_nbd_ml () =
   generate_header OCamlStyle;
 
   pr "\
-exception Error of string * int
+exception Error of string * Unix.error option
 exception Closed of string
 type cookie = int64
 
 (* Give the exceptions names so that they can be raised from the C code. *)
 let () =
-  Callback.register_exception \"nbd_internal_ocaml_error\" (Error (\"\", 0));
+  Callback.register_exception \"nbd_internal_ocaml_error\" (Error (\"\", None));
   Callback.register_exception \"nbd_internal_ocaml_closed\" (Closed \"\")
 
 ";
@@ -322,6 +331,9 @@ module Buffer = struct
   external size : t -> int = \"nbd_internal_ocaml_buffer_size\"
 end
 
+external errno_of_unix_error : Unix.error -> int =
+    \"nbd_internal_code_of_unix_error\" [@@noalloc]
+
 type t
 
 external create : unit -> t = \"nbd_internal_ocaml_nbd_create\"
@@ -689,9 +701,11 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
     | SizeT n ->
        pr "  size_t %s = Int_val (%sv);\n" n n
     | SockAddrAndLen (n, len) ->
-       pr "  const struct sockaddr *%s;\n" n;
+       pr "  struct sockaddr_storage %s_storage;\n" n;
+       pr "  struct sockaddr *%s = (struct sockaddr *) &%s_storage;\n" n n;
        pr "  socklen_t %s;\n" len;
-       pr "  abort ();\n" (* XXX *)
+       pr "  nbd_internal_unix_sockaddr_to_sa (%sv, &%s_storage, &%s);\n"
+         n n len
     | StringList n ->
        pr "  char **%s = (char **) nbd_internal_ocaml_string_list (%sv);\n" n n
     | UInt n | UIntPtr n ->
