@@ -293,7 +293,6 @@ STATE_MACHINE {
     type = be16toh (h->sbuf.sr.structured_reply.type);
 
     assert (cmd); /* guaranteed by CHECK */
-    error = nbd_internal_errno_of_nbd_error (error);
 
     /* The spec requires the server to send a non-zero error */
     if (error == NBD_SUCCESS) {
@@ -332,7 +331,7 @@ STATE_MACHINE {
     if (cmd->error == 0)
       cmd->error = error;
 
-    SET_NEXT_STATE(%FINISH);
+    SET_NEXT_STATE (%FINISH);
   }
   return 0;
 
@@ -451,7 +450,7 @@ STATE_MACHINE {
           cmd->error = error ? error : EPROTO;
     }
 
-    SET_NEXT_STATE(%FINISH);
+    SET_NEXT_STATE (%FINISH);
   }
   return 0;
 
@@ -460,7 +459,6 @@ STATE_MACHINE {
   uint32_t length;
   size_t i;
   uint32_t context_id;
-  struct meta_context *meta_context;
 
   switch (recv_into_rbuf (h)) {
   case -1: SET_NEXT_STATE (%.DEAD); return 0;
@@ -485,18 +483,16 @@ STATE_MACHINE {
 
     /* Look up the context ID. */
     context_id = h->bs_entries[0];
-    for (meta_context = h->meta_contexts;
-         meta_context;
-         meta_context = meta_context->next)
-      if (context_id == meta_context->context_id)
+    for (i = 0; i < h->meta_contexts.len; ++i)
+      if (context_id == h->meta_contexts.ptr[i].context_id)
         break;
 
-    if (meta_context) {
+    if (i < h->meta_contexts.len) {
       /* Call the caller's extent function. */
       int error = cmd->error;
 
       if (CALL_CALLBACK (cmd->cb.fn.extent,
-                         meta_context->name, cmd->offset,
+                         h->meta_contexts.ptr[i].name, cmd->offset,
                          &h->bs_entries[1], (length-4) / 4,
                          &error) == -1)
         if (cmd->error == 0)
@@ -507,7 +503,7 @@ STATE_MACHINE {
       debug (h, "server sent unexpected meta context ID %" PRIu32,
              context_id);
 
-    SET_NEXT_STATE(%FINISH);
+    SET_NEXT_STATE (%FINISH);
   }
   return 0;
 
