@@ -20,6 +20,9 @@
 open Printf
 open Unix
 
+type location = string * int
+let noloc = ("", 0)
+
 let failwithf fs = ksprintf failwith fs
 
 let rec filter_map f = function
@@ -121,34 +124,40 @@ let cspan str reject =
   in
   loop 0
 
-(* Last output column. *)
-let col = ref 0
+(* Current output line and column. *)
+let lineno = ref 1 and col = ref 0
 
 type chan = NoOutput | OutChannel of out_channel | Buffer of Buffer.t
 let chan = ref NoOutput
 let pr fs =
   ksprintf (
     fun str ->
-      (* Maintain the current output column.  We can simply do this
-       * by counting backwards from the end of the string until we
-       * reach a \n (or the beginning).  The number we count is
-       * the new column.  This only works for 7 bit ASCII but
-       * that's enough for what we need this for.
+      (* Maintain the current output row & column.  This only
+       * works for 7 bit ASCII but that's enough for what we need
+       * this for.
        *)
-      let rec loop i acc =
-        if i >= 0 then (
-          if String.unsafe_get str i = '\n' then acc
-          else loop (i-1) (acc+1)
-        )
-        else !col + acc
-      in
-      col := loop (String.length str - 1) 0;
+      for i = 0 to String.length str - 1 do
+        if String.unsafe_get str i = '\n' then (
+          col := 0;
+          incr lineno
+        ) else
+          incr col
+      done;
       match !chan with
       | NoOutput -> failwithf "use ‘output_to’ to set output"
       | OutChannel chan -> output_string chan str
       | Buffer b -> Buffer.add_string b str
   ) fs
 
+(* Wrap the output at maxcol, breaking up lines when a 'c' character
+ * occurs.  For example:
+ *   foobar = a, b, c, d, e, f, g
+ *            └── pr_wrap ',' ──┘
+ * becomes:
+ *   foobar = a, b, c,
+ *            d, e, f,
+ *            g
+ *)
 let pr_wrap ?(maxcol = 76) c code =
   (* Save the current output channel and replace it with a
    * temporary buffer while running ‘code’.  Then we wrap the
@@ -190,6 +199,13 @@ let pr_wrap ?(maxcol = 76) c code =
       *)
      pr "%s" (String.concat "\n" rest)
 
+let output_lineno () = !lineno
+let output_column () = !col
+
+let string_of_location (file, lineno) = sprintf "%s:%d" file lineno
+let line_directive_of_location (file, lineno) =
+  sprintf "#line %d \"%s\"" lineno file
+
 type comment_style =
   | CStyle | CPlusPlusStyle | HashStyle | OCamlStyle | HaskellStyle
   | PODCommentStyle
@@ -243,6 +259,7 @@ let files_equal n1 n2 =
   | i -> failwithf "%s: failed with error code %d" cmd i
 
 let output_to filename k =
+  lineno := 1; col := 0;
   let filename_new = filename ^ ".new" in
   let c = open_out filename_new in
   chan := OutChannel c;
