@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbd client library in userspace
-# Copyright (C) 2020 Red Hat Inc.
+# Copyright (C) 2020-2021 Red Hat Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,10 @@ set -x
 requires nbdkit --version
 requires nbdkit memory --version
 
+# This test requires nbdkit >= 1.12.
+minor=$( nbdkit --dump-config | grep ^version_minor | cut -d= -f2 )
+requires test $minor -ge 12
+
 out=info-text.out
 cleanup_fn rm -f $out
 
@@ -31,4 +35,5 @@ nbdkit -U - memory size=1M \
        --run '$VG nbdinfo "nbd+unix:///?socket=$unixsocket"' > $out
 cat $out
 grep "export-size: $((1024*1024))" $out
-sed -n '/contexts:/ { N; p; q }; $ q1' $out
+grep "uri: nbd+unix:///?socket=" $out
+sed -n '/contexts:/ { N; p; q; }' $out | grep .
