@@ -28,13 +28,13 @@
 
 #include <libnbd.h>
 
+#include "ansi-colours.h"
 #include "human-size.h"
-#include "vector.h"
+#include "string-vector.h"
 
 #include "nbdinfo.h"
 
-DEFINE_VECTOR_TYPE (string_vector, char *)
-
+static void show_boolean (const char *name, bool cond);
 static int collect_context (void *opaque, const char *name);
 static char *get_content (struct nbd_handle *, int64_t size);
 
@@ -68,8 +68,23 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
   if (nbd_aio_is_negotiating (nbd) &&
       nbd_opt_info (nbd) == -1 &&
       nbd_opt_go (nbd) == -1) {
-    fprintf (stderr, "%s: %s: %s\n", progname, nbd_get_export_name (nbd),
-             nbd_get_error ());
+    fprintf (stderr, "%s: %s", progname, nbd_get_error ());
+
+    char *e = nbd_get_export_name (nbd);
+    if (e) {
+      if (e[0] == '\0')
+        fprintf (stderr, " for the default export");
+      else
+        fprintf (stderr, " for export: %s", e);
+    }
+    free (e);
+    fprintf (stderr, "\n");
+
+    if (!list_all)
+      fprintf (stderr, "%s: suggestion: "
+               "to list all exports on the server, use --list\n",
+               progname);
+
     return false;
   }
   size = nbd_get_size (nbd);
@@ -80,7 +95,8 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
 
   human_size (size_str, size, &human_size_flag);
 
-  uri = nbd_get_uri (nbd);
+  if (uri_is_meaingful ())
+    uri = nbd_get_uri (nbd);
 
   /* Prefer the server's version of the name, if available */
   export_name = nbd_get_canonical_export_name (nbd);
@@ -115,6 +131,7 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
   content = get_content (nbd, size);
 
   if (!json_output) {
+    ansi_colour (ANSI_FG_BOLD_BLACK, fp);
     fprintf (fp, "export=");
     /* Might as well use the JSON function to get an escaped string here ... */
     print_json_string (export_name);
@@ -129,35 +146,36 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
       fprintf (fp, "\tcontent: %s\n", content);
     if (uri)
       fprintf (fp, "\turi: %s\n", uri);
+    ansi_restore (fp);
+    ansi_colour (ANSI_FG_GREY, fp);
     if (show_context) {
       fprintf (fp, "\tcontexts:\n");
-      for (i = 0; i < contexts.size; ++i)
+      for (i = 0; i < contexts.len; ++i)
         fprintf (fp, "\t\t%s\n", contexts.ptr[i]);
     }
     if (is_rotational >= 0)
       fprintf (fp, "\t%s: %s\n", "is_rotational",
                is_rotational ? "true" : "false");
+    ansi_restore (fp);
     if (is_read_only >= 0)
       fprintf (fp, "\t%s: %s\n", "is_read_only",
                is_read_only ? "true" : "false");
     if (can_cache >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_cache", can_cache ? "true" : "false");
+      show_boolean ("can_cache", can_cache);
     if (can_df >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_df", can_df ? "true" : "false");
+      show_boolean ("can_df", can_df);
     if (can_fast_zero >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_fast_zero",
-               can_fast_zero ? "true" : "false");
+      show_boolean ("can_fast_zero", can_fast_zero);
     if (can_flush >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_flush", can_flush ? "true" : "false");
+      show_boolean ("can_flush", can_flush);
     if (can_fua >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_fua", can_fua ? "true" : "false");
+      show_boolean ("can_fua", can_fua);
     if (can_multi_conn >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_multi_conn",
-               can_multi_conn ? "true" : "false");
+      show_boolean ("can_multi_conn", can_multi_conn);
     if (can_trim >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_trim", can_trim ? "true" : "false");
+      show_boolean ("can_trim", can_trim);
     if (can_zero >= 0)
-      fprintf (fp, "\t%s: %s\n", "can_zero", can_zero ? "true" : "false");
+      show_boolean ("can_zero", can_zero);
     if (block_minimum > 0)
       fprintf (fp, "\t%s: %" PRId64 "\n", "block_size_minimum", block_minimum);
     if (block_preferred > 0)
@@ -195,10 +213,10 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
 
     if (show_context) {
       fprintf (fp, "\t\"contexts\": [\n");
-      for (i = 0; i < contexts.size; ++i) {
+      for (i = 0; i < contexts.len; ++i) {
         fprintf (fp, "\t\t");
         print_json_string (contexts.ptr[i]);
-        if (i+1 != contexts.size)
+        if (i+1 != contexts.len)
           fputc (',', fp);
         fputc ('\n', fp);
       }
@@ -265,6 +283,18 @@ show_one_export (struct nbd_handle *nbd, const char *desc,
   return true;
 }
 
+/* Used for displaying booleans in non-JSON output. */
+static void
+show_boolean (const char *name, bool cond)
+{
+  if (cond)
+    ansi_colour (ANSI_FG_GREEN, fp);
+  else
+    ansi_colour (ANSI_FG_RED, fp);
+  fprintf (fp, "\t%s: %s\n", name, cond ? "true" : "false");
+  ansi_restore (fp);
+}
+
 static int
 collect_context (void *opaque, const char *name)
 {
