@@ -28,10 +28,13 @@
 
 #ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
+#ifdef HAVE_GNUTLS_SOCKET_H
+#include <gnutls/socket.h>
+#endif
 #endif
 
 #include "internal.h"
-#include "vector.h"
+#include "nbdkit-string.h"
 
 int
 nbd_unlocked_set_tls (struct nbd_handle *h, int tls)
@@ -108,8 +111,6 @@ nbd_unlocked_set_tls_username (struct nbd_handle *h, const char *username)
   return 0;
 }
 
-DEFINE_VECTOR_TYPE (string, char)
-
 char *
 nbd_unlocked_get_tls_username (struct nbd_handle *h)
 {
@@ -141,17 +142,19 @@ nbd_unlocked_get_tls_username (struct nbd_handle *h)
   }
 
   for (;;) {
-    if (getlogin_r (str.ptr, str.alloc) == 0) {
-      return str.ptr;
-    }
-    else if (errno != ERANGE) {
-      set_error (errno, "getlogin_r");
+    /* Increase capacity (str.cap starts at 0) */
+    if (string_reserve (&str, 16) == -1) {
+      set_error (errno, "realloc");
       free (str.ptr);
       return NULL;
     }
-    /* Try again with a larger buffer. */
-    if (string_reserve (&str, str.alloc == 0 ? 16 : str.alloc * 2) == -1) {
-      set_error (errno, "realloc");
+
+    if (getlogin_r (str.ptr, str.cap) == 0) {
+      return str.ptr;
+    }
+
+    if (errno != ERANGE) {
+      set_error (errno, "getlogin_r");
       free (str.ptr);
       return NULL;
     }
@@ -186,6 +189,22 @@ tls_recv (struct nbd_handle *h, struct socket *sock, void *buf, size_t len)
       errno = EAGAIN;
       return -1;
     }
+    if (h->tls_shut_writes &&
+        (r == GNUTLS_E_PULL_ERROR || r == GNUTLS_E_PREMATURE_TERMINATION)) {
+      /* qemu-nbd doesn't call gnutls_bye to cleanly shut down the
+       * connection after we send NBD_CMD_DISC, instead it simply
+       * closes the connection.  On the client side we see
+       * "gnutls_record_recv: The TLS connection was non-properly
+       * terminated" or "gnutls_record_recv: Error in the pull
+       * function.".
+       *
+       * If we see these errors after we shut down the write side
+       * (h->tls_shut_writes), which happens after we have sent
+       * NBD_CMD_DISC on the wire, downgrade them to a debug message.
+       */
+      debug (h, "gnutls_record_recv: %s", gnutls_strerror (r));
+      return 0; /* EOF */
+    }
     set_error (0, "gnutls_record_recv: %s", gnutls_strerror (r));
     errno = EIO;
     return -1;
@@ -233,6 +252,7 @@ tls_shut_writes (struct nbd_handle *h, struct socket *sock)
     return false;
   if (r != 0)
     debug (h, "ignoring gnutls_bye failure: %s", gnutls_strerror (r));
+  h->tls_shut_writes = true;
   return sock->u.tls.oldsock->ops->shut_writes (h, sock->u.tls.oldsock);
 }
 
@@ -586,8 +606,13 @@ nbd_internal_crypto_create_session (struct nbd_handle *h,
   gnutls_session_t session;
   gnutls_psk_client_credentials_t pskcreds = NULL;
   gnutls_certificate_credentials_t xcreds = NULL;
+  unsigned init_flags;
 
-  err = gnutls_init (&session, GNUTLS_CLIENT|GNUTLS_NONBLOCK);
+  init_flags = GNUTLS_CLIENT | GNUTLS_NONBLOCK;
+#ifdef GNUTLS_NO_SIGNAL
+  init_flags |= GNUTLS_NO_SIGNAL;
+#endif
+  err = gnutls_init (&session, init_flags);
   if (err < 0) {
     set_error (errno, "gnutls_init: %s", gnutls_strerror (err));
     return NULL;
@@ -703,15 +728,36 @@ nbd_internal_crypto_debug_tls_enabled (struct nbd_handle *h)
     const gnutls_cipher_algorithm_t cipher = gnutls_cipher_get (session);
     const gnutls_kx_algorithm_t kx = gnutls_kx_get (session);
     const gnutls_mac_algorithm_t mac = gnutls_mac_get (session);
+#ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+    const char *ktls_status;
+    gnutls_transport_ktls_enable_flags_t ktls_enabled;
+#else
+    const char *ktls_status = "disabled";
+#endif
+
+#ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+    ktls_enabled = gnutls_transport_is_ktls_enabled (session);
+    switch (ktls_enabled) {
+    case GNUTLS_KTLS_RECV: ktls_status = "enabled receive only"; break;
+    case GNUTLS_KTLS_SEND: ktls_status = "enabled send only"; break;
+    case GNUTLS_KTLS_DUPLEX: ktls_status = "enabled"; break;
+    default:
+      if ((int) ktls_enabled == 0)
+        ktls_status = "disabled";
+      else
+        ktls_status = "unknown";
+    }
+#endif
 
     debug (h,
            "connection is using TLS: "
-           "cipher %s (%zu bits) key exchange %s mac %s (%zu bits)",
+           "cipher %s (%zu bits) key exchange %s mac %s (%zu bits) kTLS %s",
            gnutls_cipher_get_name (cipher),
            8 * gnutls_cipher_get_key_size (cipher),
            gnutls_kx_get_name (kx),
            gnutls_mac_get_name (mac),
-           8 * gnutls_mac_get_key_size (mac)
+           8 * gnutls_mac_get_key_size (mac),
+           ktls_status
            );
   }
 }
