@@ -64,10 +64,15 @@
  */
 #define MAX_REQUEST_SIZE (64 * 1024 * 1024)
 
-struct meta_context;
 struct socket;
 struct command;
 
+struct meta_context {
+  char *name;                   /* Name of meta context. */
+  uint32_t context_id;          /* Context ID negotiated with the server. */
+};
+DEFINE_VECTOR_TYPE(meta_vector, struct meta_context);
+
 struct export {
   char *name;
   char *description;
@@ -174,8 +179,8 @@ struct nbd_handle {
 
   bool structured_replies;      /* If we negotiated NBD_OPT_STRUCTURED_REPLY */
 
-  /* Linked list of negotiated metadata contexts. */
-  struct meta_context *meta_contexts;
+  /* Vector of negotiated metadata contexts. */
+  meta_vector meta_contexts;
 
   /* The socket or a wrapper if using GnuTLS. */
   struct socket *sock;
@@ -310,12 +315,6 @@ struct nbd_handle {
   bool tls_shut_writes;         /* Used by lib/crypto.c to track disconnect. */
 };
 
-struct meta_context {
-  struct meta_context *next;    /* Linked list. */
-  char *name;                   /* Name of meta context. */
-  uint32_t context_id;          /* Context ID negotiated with the server. */
-};
-
 struct socket_ops {
   ssize_t (*recv) (struct nbd_handle *h,
                    struct socket *sock, void *buf, size_t len);
@@ -376,29 +375,44 @@ struct command {
   } while (0)
 
 /* aio.c */
-extern void nbd_internal_retire_and_free_command (struct command *);
+extern void nbd_internal_retire_and_free_command (struct command *)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* connect.c */
-extern int nbd_internal_wait_until_connected (struct nbd_handle *h);
+extern int nbd_internal_wait_until_connected (struct nbd_handle *h)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* crypto.c */
-extern struct socket *nbd_internal_crypto_create_session (struct nbd_handle *, struct socket *oldsock);
-extern bool nbd_internal_crypto_is_reading (struct nbd_handle *);
-extern int nbd_internal_crypto_handshake (struct nbd_handle *);
-extern void nbd_internal_crypto_debug_tls_enabled (struct nbd_handle *);
+extern struct socket *nbd_internal_crypto_create_session (struct nbd_handle *, struct socket *oldsock)
+  LIBNBD_ATTRIBUTE_NONNULL(1, 2);
+extern bool nbd_internal_crypto_is_reading (struct nbd_handle *)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
+extern int nbd_internal_crypto_handshake (struct nbd_handle *)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
+extern void nbd_internal_crypto_debug_tls_enabled (struct nbd_handle *)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* debug.c */
-extern void nbd_internal_debug (struct nbd_handle *h, const char *fs, ...);
-#define debug(h, fs, ...)                               \
-  do {                                                  \
-    if_debug ((h))                                      \
-      nbd_internal_debug ((h), (fs), ##__VA_ARGS__);    \
+extern void nbd_internal_debug (struct nbd_handle *h, const char *context,
+                                const char *fs, ...)
+  LIBNBD_ATTRIBUTE_NONNULL(1, 3);
+#define debug(h, fs, ...)                                   \
+  do {                                                      \
+    if_debug ((h))                                          \
+      nbd_internal_debug ((h), NULL, (fs), ##__VA_ARGS__);  \
+  } while (0)
+#define debug_direct(h, c, fs, ...)                       \
+  do {                                                    \
+    if_debug ((h))                                        \
+      nbd_internal_debug ((h), (c), (fs), ##__VA_ARGS__); \
   } while (0)
 
 /* errors.c */
-extern void nbd_internal_set_error_context (const char *context);
+extern void nbd_internal_set_error_context (const char *context)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 extern const char *nbd_internal_get_error_context (void);
-extern void nbd_internal_set_last_error (int errnum, char *error);
+extern void nbd_internal_set_last_error (int errnum, char *error)
+  LIBNBD_ATTRIBUTE_NONNULL(2);
 #define set_error(errnum, fs, ...)                                      \
   do {                                                                  \
     int _e = (errnum);                                                  \
@@ -416,12 +430,15 @@ extern void nbd_internal_set_last_error (int errnum, char *error);
   } while (0)
 
 /* flags.c */
-extern void nbd_internal_reset_size_and_flags (struct nbd_handle *h);
+extern void nbd_internal_reset_size_and_flags (struct nbd_handle *h)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 extern int nbd_internal_set_size_and_flags (struct nbd_handle *h,
                                             uint64_t exportsize,
-                                            uint16_t eflags);
+                                            uint16_t eflags)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 extern int nbd_internal_set_block_size (struct nbd_handle *h, uint32_t min,
-                                        uint32_t pref, uint32_t max);
+                                        uint32_t pref, uint32_t max)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* is-state.c */
 extern bool nbd_internal_is_state_created (enum state state);
@@ -433,7 +450,8 @@ extern bool nbd_internal_is_state_dead (enum state state);
 extern bool nbd_internal_is_state_closed (enum state state);
 
 /* opt.c */
-extern void nbd_internal_free_option (struct nbd_handle *h);
+extern void nbd_internal_free_option (struct nbd_handle *h)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* protocol.c */
 extern int nbd_internal_errno_of_nbd_error (uint32_t error);
@@ -444,15 +462,18 @@ extern int64_t nbd_internal_command_common (struct nbd_handle *h,
                                             uint16_t flags, uint16_t type,
                                             uint64_t offset, uint64_t count,
                                             int count_err, void *data,
-                                            struct command_cb *cb);
+                                            struct command_cb *cb)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 
 /* socket.c */
 struct socket *nbd_internal_socket_create (int fd);
 
 /* states.c */
 extern void nbd_internal_abort_commands (struct nbd_handle *h,
-                                         struct command **list);
-extern int nbd_internal_run (struct nbd_handle *h, enum external_event ev);
+                                         struct command **list)
+  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+extern int nbd_internal_run (struct nbd_handle *h, enum external_event ev)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
 extern const char *nbd_internal_state_short_string (enum state state);
 extern enum state_group nbd_internal_state_group (enum state state);
 extern enum state_group nbd_internal_state_group_parent (enum state_group group);
@@ -463,13 +484,23 @@ extern int nbd_internal_aio_get_direction (enum state state);
 #define get_public_state(h) ((h)->public_state)
 
 /* utils.c */
-extern void nbd_internal_hexdump (const void *data, size_t len, FILE *fp);
-extern int nbd_internal_set_argv (string_vector *v, char **argv);
-extern const char *nbd_internal_fork_safe_itoa (long v, char *buf, size_t len);
-extern void nbd_internal_fork_safe_perror (const char *s);
-extern char *nbd_internal_printable_buffer (const void *buf, size_t count);
-extern char *nbd_internal_printable_string (const char *str);
-extern char *nbd_internal_printable_string_list (char **list);
+extern void nbd_internal_hexdump (const void *data, size_t len, FILE *fp)
+  LIBNBD_ATTRIBUTE_NONNULL(1,3);
+extern int nbd_internal_copy_string_list (string_vector *v, char **in)
+  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+extern int nbd_internal_set_argv (struct nbd_handle *h, char **argv)
+  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+extern const char *nbd_internal_fork_safe_itoa (long v, char *buf, size_t len)
+  LIBNBD_ATTRIBUTE_NONNULL(2);
+extern void nbd_internal_fork_safe_perror (const char *s)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
+extern char *nbd_internal_printable_buffer (const void *buf, size_t count)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
+extern char *nbd_internal_printable_string (const char *str)
+  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(free)
+  LIBNBD_ATTRIBUTE_NONNULL(1);
+extern char *nbd_internal_printable_string_list (char **list)
+  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(free);
 
 /* These are wrappers around socket(2) and socketpair(2).  They
  * always set SOCK_CLOEXEC.  nbd_internal_socket can set SOCK_NONBLOCK
@@ -478,6 +509,7 @@ extern char *nbd_internal_printable_string_list (char **list);
 extern int nbd_internal_socket (int domain, int type, int protocol,
                                 bool nonblock);
 extern int nbd_internal_socketpair (int domain, int type, int protocol,
-                                    int *fds);
+                                    int *fds)
+  LIBNBD_ATTRIBUTE_NONNULL(4);
 
 #endif /* LIBNBD_INTERNAL_H */
