@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2022 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -53,16 +53,17 @@ nbd_internal_hexdump (const void *data, size_t len, FILE *fp)
   }
 }
 
-/* Replace the string_vector with a deep copy of argv (including final NULL). */
+/* Replace a string_vector with a deep copy of in (including final NULL). */
 int
-nbd_internal_set_argv (string_vector *v, char **argv)
+nbd_internal_copy_string_list (string_vector *v, char **in)
 {
   size_t i;
 
-  string_vector_reset (v);
+  assert (in);
+  assert (v->ptr == NULL);
 
-  for (i = 0; argv[i] != NULL; ++i) {
-    char *copy = strdup (argv[i]);
+  for (i = 0; in[i] != NULL; ++i) {
+    char *copy = strdup (in[i]);
     if (copy == NULL)
       return -1;
     if (string_vector_append (v, copy) == -1) {
@@ -74,6 +75,35 @@ nbd_internal_set_argv (string_vector *v, char **argv)
   return string_vector_append (v, NULL);
 }
 
+/* Store argv into h, or diagnose an error on failure. */
+int
+nbd_internal_set_argv (struct nbd_handle *h, char **argv)
+{
+  /* This should never be NULL.  The generator adds code to each
+   * StringList call in lib/api.c to check this and return an error.
+   */
+  assert (argv);
+
+  /* Because this function is only called from functions that take
+   * argv-style lists of strings (such as nbd_connect_command) we can
+   * check here that the command name is present.
+   */
+  if (argv[0] == NULL) {
+    set_error (EINVAL, "missing command name in argv list");
+    return -1;
+  }
+
+  string_vector_iter (&h->argv, (void *) free);
+  string_vector_reset (&h->argv);
+
+  if (nbd_internal_copy_string_list (&h->argv, argv) == -1) {
+    set_error (errno, "realloc");
+    return -1;
+  }
+
+  return 0;
+}
+
 /* Like sprintf ("%ld", v), but safe to use between fork and exec.  Do
  * not use this function in any other context.
  *
@@ -247,13 +277,17 @@ nbd_internal_printable_string_list (char **list)
   if (fp == NULL)
     return NULL;
 
-  fprintf (fp, "[");
-  for (i = 0; list[i] != NULL; ++i) {
-    if (i > 0)
-      fprintf (fp, ", ");
-    printable_string (list[i], fp);
+  if (list == NULL)
+    fprintf (fp, "NULL");
+  else {
+    fprintf (fp, "[");
+    for (i = 0; list[i] != NULL; ++i) {
+      if (i > 0)
+        fprintf (fp, ", ");
+      printable_string (list[i], fp);
+    }
+    fprintf (fp, "]");
   }
-  fprintf (fp, "]");
   fclose (fp);
 
   return s;
@@ -289,7 +323,7 @@ int nbd_internal_socket(int domain,
     return -1;
 
   if (fcntl (fd, F_SETFD, FD_CLOEXEC) == -1) {
-    close(fd);
+    close (fd);
     return -1;
   }
 
@@ -297,7 +331,7 @@ int nbd_internal_socket(int domain,
     flags = fcntl (fd, F_GETFL, 0);
     if (flags == -1 ||
         fcntl (fd, F_SETFL, flags|O_NONBLOCK) == -1) {
-      close(fd);
+      close (fd);
       return -1;
     }
   }
@@ -328,8 +362,8 @@ nbd_internal_socketpair (int domain, int type, int protocol, int *fds)
   if (ret == 0) {
     for (i = 0; i < 2; i++) {
       if (fcntl (fds[i], F_SETFD, FD_CLOEXEC) == -1) {
-        close(fds[0]);
-        close(fds[1]);
+        close (fds[0]);
+        close (fds[1]);
         return -1;
       }
     }
