@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright (C) 2013-2022 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/socket.h>
+#include <assert.h>
 
 #include <caml/alloc.h>
 #include <caml/callback.h>
@@ -30,16 +32,25 @@
 #include <caml/memory.h>
 #include <caml/mlvalues.h>
 #include <caml/printexc.h>
+#include <caml/socketaddr.h>
+#include <caml/unixsupport.h>
 
 #include <libnbd.h>
 
 #include "nbd-c.h"
 
+/* NB: @@noalloc function */
+value
+nbd_internal_code_of_unix_error (value error)
+{
+  return Val_int (code_of_unix_error (error));
+}
+
 void
 nbd_internal_ocaml_raise_error (void)
 {
   CAMLparam0 ();
-  CAMLlocal1 (sv);
+  CAMLlocal3 (sv, errv, ev);
   value v[2];
   const char *msg;
   int err;
@@ -52,8 +63,18 @@ nbd_internal_ocaml_raise_error (void)
   else
     sv = caml_copy_string ("no error message available");
 
+  /* Convert raw C errno to OCaml Unix.error, if available. */
+  if (err == 0) {
+    errv = Val_int (0); /* None */
+  }
+  else {
+    ev = unix_error_of_code (err);
+    errv = caml_alloc (1, 0);   /* Some ev */
+    Field (errv, 0) = ev;
+  }
+
   v[0] = sv;
-  v[1] = Val_int (err);
+  v[1] = errv;
   caml_raise_with_args (*caml_named_value ("nbd_internal_ocaml_error"),
                         2, v);
   CAMLnoreturn;
@@ -97,7 +118,7 @@ nbd_internal_ocaml_string_list (value ssv)
 }
 
 value
-nbd_internal_ocaml_alloc_int32_array (uint32_t *a, size_t len)
+nbd_internal_ocaml_alloc_int64_from_uint32_array (uint32_t *a, size_t len)
 {
   CAMLparam0 ();
   CAMLlocal2 (v, rv);
@@ -105,17 +126,42 @@ nbd_internal_ocaml_alloc_int32_array (uint32_t *a, size_t len)
 
   rv = caml_alloc (len, 0);
   for (i = 0; i < len; ++i) {
-    v = caml_copy_int32 (a[i]);
+    v = caml_copy_int64 (a[i]);
     Store_field (rv, i, v);
   }
 
   CAMLreturn (rv);
 }
 
-/* Common code when an exception is raised in an OCaml callback and
- * the wrapper has to deal with it.  Callbacks are not supposed to
- * raise exceptions, so we print it.  We also handle Assert_failure
- * specially by abort()-ing.
+/* Convert a Unix.sockaddr to a C struct sockaddr. */
+void
+nbd_internal_unix_sockaddr_to_sa (value sockaddrv,
+                                  struct sockaddr_storage *ss,
+                                  socklen_t *len)
+{
+  CAMLparam1 (sockaddrv);
+  union sock_addr_union sa_u;
+  socklen_param_type sl; /* this is really an int or socklen_t */
+
+  memset (ss, 0, sizeof *ss);
+
+#ifdef HAVE_CAML_UNIX_GET_SOCKADDR
+  caml_unix_get_sockaddr (sockaddrv, &sa_u, &sl);
+#else
+  /* OCaml <= 4.14 exports this unnamespaced symbol. */
+  get_sockaddr (sockaddrv, &sa_u, &sl);
+#endif
+  assert (sl <= sizeof *ss);
+  memcpy (ss, &sa_u, sl);
+  *len = sl;
+
+  CAMLreturn0;
+}
+
+/* Common code when an exception is raised in an OCaml callback.
+ *
+ * We handle Assert_failure specially by abort()-ing.  Other
+ * exceptions are printed to make sure we don't lose information.
  */
 void
 nbd_internal_ocaml_exception_in_wrapper (const char *cbname, value rv)
@@ -126,6 +172,10 @@ nbd_internal_ocaml_exception_in_wrapper (const char *cbname, value rv)
   char *s;
 
   exn = Extract_exception (rv);
+  s = caml_format_exception (exn);
+  fprintf (stderr,
+           "libnbd: %s: uncaught OCaml exception: %s\n", cbname, s);
+  free (s);
 
   /* For how we're getting the exception name, see:
    * https://github.com/libguestfs/libguestfs/blob/5d94be2583d557cfc7f8a8cfee7988abfa45a3f8/daemon/daemon-c.c#L40
@@ -135,11 +185,6 @@ nbd_internal_ocaml_exception_in_wrapper (const char *cbname, value rv)
   else
     exn_name = String_val (Field (Field (exn, 0), 0));
 
-  s = caml_format_exception (exn);
-  fprintf (stderr,
-           "libnbd: %s: uncaught OCaml exception: %s\n", cbname, s);
-  free (s);
-
   /* If the exception is fatal (like Assert_failure) then abort. */
   if (exn_name && strcmp (exn_name, "Assert_failure") == 0)
     abort ();
