@@ -18,8 +18,11 @@
  *)
 
 let () =
-  let nbd = NBD.create () in
-  NBD.connect_command nbd
-                      ["nbdkit"; "-s"; "--exit-with-parent"; "-v"; "null"]
+  NBD.with_handle (
+    fun nbd ->
+      NBD.connect_command nbd
+                          ["nbdkit"; "-s"; "--exit-with-parent"; "-v";
+                           "null"]
+  )
 
 let () = Gc.compact ()
