@@ -17,18 +17,24 @@
 
 import nbd
 
-h = nbd.NBD ()
+h = nbd.NBD()
 
 messages = []
+
+
 def f(id, context, msg):
     global messages
 
     assert id == 42
-    messages.append (msg)
+    messages.append(msg)
 
-h.set_debug_callback (lambda *args: f (42, *args))
 
-h.connect_command (["nbdkit", "-s", "--exit-with-parent", "null"])
-h.shutdown ()
+h.set_debug_callback(lambda *args: f(42, *args))
 
-print ("%r" % messages)
+h.connect_command(["nbdkit", "-s", "--exit-with-parent", "null"])
+h.shutdown()
+
+# Try to trigger garbage collection of h
+h = None
+
+print("%r" % messages)
